@@ -1,9 +1,14 @@
 """Tibber home"""
+from __future__ import annotations
+
+import asyncio
 import datetime as dt
 import logging
-from typing import Callable, Optional
+import random
+from typing import TYPE_CHECKING, Callable
 
 from dateutil.parser import parse
+from gql import gql
 
 from .const import RESOLUTION_HOURLY
 from .gql_queries import (
@@ -16,6 +21,9 @@ from .gql_queries import (
     UPDATE_INFO_PRICE,
 )
 
+if TYPE_CHECKING:
+    from . import Tibber
+
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -25,11 +33,11 @@ class HourlyData:
     # pylint: disable=too-few-public-methods
     def __init__(self, production: bool = False):
         self.is_production: bool = production
-        self.month_energy: Optional[float] = None
-        self.month_money: Optional[float] = None
-        self.peak_hour: Optional[float] = None
-        self.peak_hour_time: Optional[dt.datetime] = None
-        self.last_data_timestamp: Optional[dt.datetime] = None
+        self.month_energy: float | None = None
+        self.month_money: float | None = None
+        self.peak_hour: float | None = None
+        self.peak_hour_time: dt.datetime | None = None
+        self.last_data_timestamp: dt.datetime | None = None
         self.data: list[dict] = []
 
 
@@ -38,7 +46,7 @@ class TibberHome:
 
     # pylint: disable=too-many-instance-attributes, too-many-public-methods
 
-    def __init__(self, home_id: str, tibber_control):
+    def __init__(self, home_id: str, tibber_control: Tibber) -> None:
         """Initialize the Tibber home class.
 
         :param home_id: The ID of the home.
@@ -47,14 +55,13 @@ class TibberHome:
         """
         self._tibber_control = tibber_control
         self._home_id: str = home_id
-        self._current_price_total: Optional[float] = None
+        self._current_price_total: float | None = None
         self._current_price_info: dict[str, float] = {}
         self._price_info: dict[str, float] = {}
         self._level_info: dict[str, str] = {}
         self._rt_power: list[tuple[dt.datetime, float]] = []
         self.info: dict[str, dict] = {}
-        self._subscription_id: Optional[str] = None
-        self.last_data_timestamp: Optional[dt.datetime] = None
+        self.last_data_timestamp: dt.datetime | None = None
 
         self._hourly_consumption_data: HourlyData = HourlyData()
         self._hourly_production_data: HourlyData = HourlyData(production=True)
@@ -115,7 +122,7 @@ class TibberHome:
         _month_energy = 0
         _month_money = 0
         _month_hour_max_month_hour_energy = 0
-        _month_hour_max_month_hour: Optional[dt.datetime] = None
+        _month_hour_max_month_hour: dt.datetime | None = None
 
         for node in hourly_data.data:
             _time = parse(node["from"])
@@ -151,27 +158,27 @@ class TibberHome:
         return await self._fetch_data(self._hourly_production_data)
 
     @property
-    def month_cons(self) -> Optional[float]:
+    def month_cons(self) -> float | None:
         """Get consumption for current month."""
         return self._hourly_consumption_data.month_energy
 
     @property
-    def month_cost(self) -> Optional[float]:
+    def month_cost(self) -> float | None:
         """Get total cost for current month."""
         return self._hourly_consumption_data.month_money
 
     @property
-    def peak_hour(self) -> Optional[float]:
+    def peak_hour(self) -> float | None:
         """Get consumption during peak hour for the current month."""
         return self._hourly_consumption_data.peak_hour
 
     @property
-    def peak_hour_time(self) -> Optional[dt.datetime]:
+    def peak_hour_time(self) -> dt.datetime | None:
         """Get the time for the peak consumption during the current month."""
         return self._hourly_consumption_data.peak_hour_time
 
     @property
-    def last_cons_data_timestamp(self) -> Optional[dt.datetime]:
+    def last_cons_data_timestamp(self) -> dt.datetime | None:
         """Get last consumption data timestampt."""
         return self._hourly_consumption_data.last_data_timestamp
 
@@ -254,7 +261,7 @@ class TibberHome:
                     self.last_data_timestamp = parse(data.get("startsAt"))
 
     @property
-    def current_price_total(self) -> Optional[float]:
+    def current_price_total(self) -> float | None:
         """Get current price total."""
         if not self._current_price_info:
             return None
@@ -372,29 +379,22 @@ class TibberHome:
 
         :param callback: The function to call when data is received.
         """
-        if self._subscription_id is not None:
-            _LOGGER.error("Already subscribed.")
-            return
-        await self._tibber_control.rt_connect()
-        document = LIVE_SUBSCRIBE % self.home_id
 
         def _add_extra_data(data: dict) -> dict:
-            _time = parse(data["data"]["liveMeasurement"]["timestamp"]).astimezone(
+            live_data = data["data"]["liveMeasurement"]
+            _time = parse(live_data["timestamp"]).astimezone(
                 self._tibber_control.time_zone
             )
-            self._rt_power.append(
-                (_time, data["data"]["liveMeasurement"]["power"] / 1000)
-            )
             while self._rt_power and self._rt_power[0][0] < _time - dt.timedelta(
                 minutes=5
             ):
                 self._rt_power.pop(0)
-            current_hour = data["data"]["liveMeasurement"][
-                "accumulatedConsumptionLastHour"
-            ]
+
+            self._rt_power.append((_time, live_data["power"] / 1000))
+            current_hour = live_data["accumulatedConsumptionLastHour"]
             if current_hour is not None:
                 power = sum(p[1] for p in self._rt_power) / len(self._rt_power)
-                data["data"]["liveMeasurement"]["estimatedHourConsumption"] = round(
+                live_data["estimatedHourConsumption"] = round(
                     current_hour
                     + power * (3600 - (_time.minute * 60 + _time.second)) / 3600,
                     3,
@@ -407,34 +407,49 @@ class TibberHome:
                     self._hourly_consumption_data.peak_hour_time = _time
             return data
 
-        def callback_add_extra_data(data: dict) -> None:
-            """Add estimated hourly consumption."""
-            try:
-                data = _add_extra_data(data)
-            except KeyError:
-                pass
-            callback(data)
+        async def _disconnect():
+            await asyncio.sleep(30)
+            _LOGGER.debug("No data received for 30 seconds, reconnecting")
+            await self._tibber_control.rt_disconnect()
 
-        if self._tibber_control.sub_manager is not None:
-            self._subscription_id = await self._tibber_control.sub_manager.subscribe(
-                document, callback_add_extra_data
-            )
+        async def _start():
+            """Subscribe to Tibber."""
+            _retry_count = 0
+            restarter = asyncio.ensure_future(_disconnect())
+            while True:
+                try:
+                    if not self.rt_subscription_running:
+                        await self._tibber_control.sub_manager.connect_async()
+                    async for data in self._tibber_control.sub_manager.session.subscribe(
+                        gql(LIVE_SUBSCRIBE % self.home_id)
+                    ):
+                        restarter.cancel()
+                        restarter = asyncio.ensure_future(_disconnect())
+                        data = {"data": data}
+                        try:
+                            data = _add_extra_data(data)
+                        except KeyError:
+                            pass
+                        callback(data)
+                        _retry_count = 0
+                except Exception:  # pylint: disable=broad-except
+                    delay_seconds = min(
+                        random.SystemRandom().randint(1, 60) + _retry_count**2,
+                        60 * 60,
+                    )
+                    _LOGGER.error(
+                        "Tibber connection closed, will reconnect in %s seconds",
+                        delay_seconds,
+                    )
+                    _retry_count += 1
+                    await asyncio.sleep(delay_seconds)
 
-    async def rt_unsubscribe(self) -> None:
-        """Unsubscribe to Tibber real time subscription."""
-        if self._subscription_id is None:
-            _LOGGER.error("Not subscribed.")
-            return
-        if self._tibber_control.sub_manager is not None:
-            await self._tibber_control.sub_manager.unsubscribe(self._subscription_id)
+        asyncio.get_running_loop().create_task(_start())
 
     @property
     def rt_subscription_running(self) -> bool:
         """Is real time subscription running."""
-        return (
-            self._tibber_control.sub_manager is not None
-            and self._tibber_control.sub_manager.is_running is True
-        )
+        return self._tibber_control.rt_subscription_running
 
     async def get_historic_data(
         self, n_data: int, resolution: str = RESOLUTION_HOURLY, production: bool = False
@@ -467,7 +482,7 @@ class TibberHome:
     async def get_historic_price_data(
         self,
         resolution: str = RESOLUTION_HOURLY,
-    ) -> Optional[list[dict]]:
+    ) -> list[dict] | None:
         """Get historic price data.
         :param resolution: The resolution of the data. Can be HOURLY,
             DAILY, WEEKLY, MONTHLY or ANNUAL
@@ -536,4 +551,8 @@ class TibberHome:
             if now.hour >= 22 or now.hour < 6:
                 grid_price -= 12 / 100
             attr["grid_price"] = round(grid_price, 3)
+            _LOGGER.warning(
+                "Grid price attribute is deprecated and will be removed. "
+                "Use https://github.com/Danielhiversen/home_assistant_glitre"
+            )
         return attr
